#include <iostream>
#include <memory>
#include <boost/program_options.hpp>
#include <boost/lexical_cast.hpp>
#include "artdaq/DAQdata/Globals.hh"
#include "artdaq/Application/TaskType.hh"
#include "artdaq/Application/EventBuilderApp.hh"
#include "artdaq/Application/LoadParameterSet.hh"
#include "artdaq/ExternalComms/MakeCommanderPlugin.hh"
#include "artdaq-mpich-plugin/Application/MPISentry.hh"
#include "artdaq/BuildInfo/GetPackageBuildInfo.hh"
#include "cetlib_except/exception.h"

int main(int argc, char* argv[])
{
	// initialization

	int const wanted_threading_level{ MPI_THREAD_MULTIPLE };
	MPI_Comm local_group_comm;
	std::unique_ptr<artdaq::MPISentry> mpiSentry;

	try
	{
		mpiSentry.reset(new artdaq::MPISentry(&argc, &argv, wanted_threading_level, artdaq::TaskType::EventBuilderTask, local_group_comm));
	}
	catch (cet::exception& errormsg)
	{
		TLOG_ERROR("EventBuilderMain") << errormsg ;
		TLOG_ERROR("EventBuilderMain") << "MPISentry error encountered in EventBuilderMain; exiting..." ;
		throw errormsg;
	}

	fhicl::ParameterSet config = LoadParameterSet(argc, argv);
	app_name = config.get<std::string>("application_name", "EventBuilder");
	std::string mf_app_name = artdaq::setMsgFacAppName(app_name, config.get<int>("id"));
	artdaq::configureMessageFacility(mf_app_name.c_str());
	TLOG_DEBUG(app_name + "Main") << "Setting application name to " << mf_app_name ;


	TLOG_INFO(app_name + "Main") << "artdaq version " <<
		artdaq::GetPackageBuildInfo::getPackageBuildInfo().getPackageVersion()
		<< ", built " <<
		artdaq::GetPackageBuildInfo::getPackageBuildInfo().getBuildTimestamp() ;

	// create the EventBuilderApp
	my_rank = mpiSentry->rank();
	artdaq::EventBuilderApp evb_app;

	TLOG_DEBUG(app_name + "Main") << "Creating EventBuilder Commander plugin" ;
	auto commander = artdaq::MakeCommanderPlugin(config, evb_app);

	TLOG_INFO(app_name + "Main") << "Running Commmander Server" ;
	commander->run_server();
	TLOG_INFO(app_name + "Main") << "Commandable Server ended, exiting..." ;
}
